import { Array } from "@siteimprove/alfa-array";
import { Result } from "@siteimprove/alfa-result";
import { Unit } from "../../unit/index.js";
import type { Numeric } from "../numeric/index.js";
import { Expression } from "./expression.js";
import type { Kind } from "./kind.js";
/**
 * @public
 */
export declare abstract class Function<T extends string = string, A extends Array<Expression> = Array<Expression>> extends Expression<T> {
    protected readonly _args: Readonly<A>;
    protected readonly _kind: Kind;
    protected constructor(type: T, args: Readonly<A>, kind: Kind);
    get args(): Readonly<A>;
    equals(value: Function<T, A>): boolean;
    equals(value: unknown): value is this;
    toJSON(): Function.JSON<T>;
}
/**
 * @public
 */
export declare namespace Function {
    interface JSON<T extends string = string> extends Expression.JSON<T> {
        arguments: Array<Expression.JSON>;
    }
    class Calculation extends Function<"calculation", [Expression]> {
        static of(expression: Expression): Calculation;
        private constructor();
        reduce<L extends Unit.Length = Unit.Length.Canonical, P extends Numeric = Numeric>(resolver: Expression.Resolver<L, P>): Expression;
        toString(): string;
    }
    function isCalculation(value: unknown): value is Calculation;
    class Max extends Function<"max", [Expression, ...Array<Expression>]> {
        static of(first: Expression, ...expressions: ReadonlyArray<Expression>): Result<Max, string>;
        private constructor();
        reduce<L extends Unit.Length = Unit.Length.Canonical, P extends Numeric = Numeric>(resolver: Expression.Resolver<L, P>): Expression;
        toString(): string;
    }
}
//# sourceMappingURL=function.d.ts.map