import type { Equatable } from "@siteimprove/alfa-equatable";
import type { Serializable } from "@siteimprove/alfa-json";
import { Parser } from "@siteimprove/alfa-parser";
import type { Predicate } from "@siteimprove/alfa-predicate";
import type { Slice } from "@siteimprove/alfa-slice";
import type * as json from "@siteimprove/alfa-json";
/**
 * @public
 */
export type Token = Token.Ident | Token.Function | Token.AtKeyword | Token.Hash | Token.String | Token.URL | Token.BadURL | Token.Delim | Token.Number | Token.Percentage | Token.Dimension | Token.Whitespace | Token.Colon | Token.Semicolon | Token.Comma | Token.OpenParenthesis | Token.CloseParenthesis | Token.OpenSquareBracket | Token.CloseSquareBracket | Token.OpenCurlyBracket | Token.CloseCurlyBracket | Token.OpenComment | Token.CloseComment;
/**
 * @public
 */
export declare namespace Token {
    type JSON = Ident.JSON | Function.JSON | AtKeyword.JSON | Hash.JSON | String.JSON | URL.JSON | BadURL.JSON | Delim.JSON | Number.JSON | Percentage.JSON | Dimension.JSON | Whitespace.JSON | Colon.JSON | Semicolon.JSON | Comma.JSON | OpenParenthesis.JSON | CloseParenthesis.JSON | OpenSquareBracket.JSON | CloseSquareBracket.JSON | OpenCurlyBracket.JSON | CloseCurlyBracket.JSON | OpenComment.JSON | CloseComment.JSON;
    class Ident implements Equatable, Serializable<Ident.JSON> {
        static of(value: string): Ident;
        private readonly _value;
        private constructor();
        get type(): "ident";
        get value(): string;
        equals(value: unknown): value is this;
        toJSON(): Ident.JSON;
        toString(): string;
    }
    namespace Ident {
        interface JSON {
            [key: string]: json.JSON;
            type: "ident";
            value: string;
        }
        function isIdent(value: unknown): value is Ident;
    }
    const ident: typeof Ident.of, isIdent: typeof Ident.isIdent;
    function parseIdent(query?: string | Predicate<Ident>): Parser<Slice<Token>, Ident, string, []>;
    class Function implements Equatable, Serializable<Function.JSON> {
        static of(value: string): Function;
        private readonly _value;
        private constructor();
        get type(): "function";
        get value(): string;
        get mirror(): CloseParenthesis;
        equals(value: unknown): value is this;
        toJSON(): Function.JSON;
        toString(): string;
    }
    namespace Function {
        interface JSON {
            [key: string]: json.JSON;
            type: "function";
            value: string;
        }
        function isFunction(value: unknown): value is Function;
    }
    const func: typeof Function.of, isFunction: typeof Function.isFunction;
    function parseFunction(query?: string | Predicate<Function>): Parser<Slice<Token>, Function, string, []>;
    class AtKeyword implements Equatable, Serializable<AtKeyword.JSON> {
        static of(value: string): AtKeyword;
        private readonly _value;
        private constructor();
        get type(): "at-keyword";
        get value(): string;
        equals(value: unknown): value is this;
        toJSON(): AtKeyword.JSON;
        toString(): string;
    }
    namespace AtKeyword {
        interface JSON {
            [key: string]: json.JSON;
            type: "at-keyword";
            value: string;
        }
    }
    const atKeyword: typeof AtKeyword.of;
    class Hash implements Equatable, Serializable<Hash.JSON> {
        static of(value: string, isIdentifier: boolean): Hash;
        private readonly _value;
        private readonly _isIdentifier;
        private constructor();
        get type(): "hash";
        get value(): string;
        get isIdentifier(): boolean;
        equals(value: unknown): value is this;
        toJSON(): Hash.JSON;
        toString(): string;
    }
    namespace Hash {
        interface JSON {
            [key: string]: json.JSON;
            type: "hash";
            value: string;
            isIdentifier: boolean;
        }
        function isHash(value: unknown): value is Hash;
    }
    const hash: typeof Hash.of, isHash: typeof Hash.isHash;
    function parseHash(predicate?: Predicate<Hash>): Parser<Slice<Token>, Hash, string, []>;
    class String implements Equatable, Serializable<String.JSON> {
        static of(value: string): String;
        private readonly _value;
        private constructor();
        get type(): "string";
        get value(): string;
        equals(value: unknown): value is this;
        toJSON(): String.JSON;
        toString(): string;
    }
    namespace String {
        interface JSON {
            [key: string]: json.JSON;
            type: "string";
            value: string;
        }
        function isString(value: unknown): value is String;
    }
    const string: typeof String.of, isString: typeof String.isString;
    function parseString(predicate?: Predicate<String>): Parser<Slice<Token>, String, string, []>;
    class URL implements Equatable, Serializable<URL.JSON> {
        static of(value: string): URL;
        private readonly _value;
        private constructor();
        get type(): "url";
        get value(): string;
        equals(value: unknown): value is this;
        toJSON(): URL.JSON;
        toString(): string;
    }
    namespace URL {
        interface JSON {
            [key: string]: json.JSON;
            type: "url";
            value: string;
        }
        function isURL(value: unknown): value is URL;
    }
    const url: typeof URL.of, isURL: typeof URL.isURL;
    function parseURL(predicate?: Predicate<URL>): Parser<Slice<Token>, URL, string, []>;
    class BadURL implements Equatable, Serializable<BadURL.JSON> {
        static of(): BadURL;
        private constructor();
        get type(): "bad-url";
        equals(value: unknown): value is this;
        toJSON(): BadURL.JSON;
        toString(): string;
    }
    namespace BadURL {
        interface JSON {
            [key: string]: json.JSON;
            type: "bad-url";
        }
    }
    const badURL: typeof BadURL.of;
    class Delim implements Equatable, Serializable<Delim.JSON> {
        private static _delims;
        static of(value: number): Delim;
        private readonly _value;
        private constructor();
        get type(): "delim";
        get value(): number;
        equals(value: unknown): value is this;
        toJSON(): Delim.JSON;
        toString(): string;
    }
    namespace Delim {
        interface JSON {
            [key: string]: json.JSON;
            type: "delim";
            value: number;
        }
        function isDelim(value: unknown): value is Delim;
    }
    const delim: typeof Delim.of, isDelim: typeof Delim.isDelim;
    function parseDelim(query?: string | number | Predicate<Delim>): Parser<Slice<Token>, Delim, string, []>;
    class Number implements Equatable, Serializable<Number.JSON> {
        static of(value: number, isInteger: boolean, isSigned: boolean): Number;
        private readonly _value;
        private readonly _isInteger;
        private readonly _isSigned;
        private constructor();
        get type(): "number";
        get value(): number;
        get isInteger(): boolean;
        get isSigned(): boolean;
        equals(value: unknown): value is this;
        toJSON(): Number.JSON;
        toString(): string;
    }
    namespace Number {
        interface JSON {
            [key: string]: json.JSON;
            type: "number";
            value: number;
            isInteger: boolean;
            isSigned: boolean;
        }
        function isNumber(value: unknown): value is Number;
    }
    const number: typeof Number.of, isNumber: typeof Number.isNumber;
    function parseNumber(predicate?: Predicate<Number>): Parser<Slice<Token>, Number, string, []>;
    class Percentage implements Equatable, Serializable<Percentage.JSON> {
        static of(value: number, isInteger: boolean): Percentage;
        private readonly _value;
        private readonly _isInteger;
        private constructor();
        get type(): "percentage";
        get value(): number;
        get isInteger(): boolean;
        equals(value: unknown): value is this;
        toJSON(): Percentage.JSON;
        toString(): string;
    }
    namespace Percentage {
        interface JSON {
            [key: string]: json.JSON;
            type: "percentage";
            value: number;
            isInteger: boolean;
        }
        function isPercentage(value: unknown): value is Percentage;
    }
    const percentage: typeof Percentage.of, isPercentage: typeof Percentage.isPercentage;
    function parsePercentage(predicate?: Predicate<Percentage>): Parser<Slice<Token>, Percentage, string, []>;
    class Dimension implements Equatable, Serializable<Dimension.JSON> {
        static of(value: number, unit: string, isInteger: boolean, isSigned: boolean): Dimension;
        private readonly _value;
        private readonly _unit;
        private readonly _isInteger;
        private readonly _isSigned;
        private constructor();
        get type(): "dimension";
        get value(): number;
        get unit(): string;
        get isInteger(): boolean;
        get isSigned(): boolean;
        equals(value: unknown): value is this;
        toJSON(): Dimension.JSON;
        toString(): string;
    }
    namespace Dimension {
        interface JSON {
            [key: string]: json.JSON;
            type: "dimension";
            value: number;
            unit: string;
            isInteger: boolean;
            isSigned: boolean;
        }
        function isDimension(value: unknown): value is Dimension;
    }
    const dimension: typeof Dimension.of, isDimension: typeof Dimension.isDimension;
    function parseDimension(predicate?: Predicate<Dimension>): Parser<Slice<Token>, Dimension, string, []>;
    class Whitespace implements Equatable, Serializable<Whitespace.JSON> {
        private static _instance;
        static of(): Whitespace;
        private constructor();
        get type(): "whitespace";
        equals(value: unknown): value is this;
        toJSON(): Whitespace.JSON;
        toString(): string;
    }
    namespace Whitespace {
        interface JSON {
            [key: string]: json.JSON;
            type: "whitespace";
        }
        function isWhitespace(value: unknown): value is Whitespace;
    }
    const whitespace: typeof Whitespace.of, isWhitespace: typeof Whitespace.isWhitespace;
    const parseWhitespace: Parser<Slice<Token>, Whitespace, string, []>;
    class Colon implements Equatable, Serializable<Colon.JSON> {
        private static _instance;
        static of(): Colon;
        private constructor();
        get type(): "colon";
        equals(value: unknown): value is this;
        toJSON(): Colon.JSON;
        toString(): string;
    }
    namespace Colon {
        interface JSON {
            [key: string]: json.JSON;
            type: "colon";
        }
        function isColon(value: unknown): value is Colon;
    }
    const colon: typeof Colon.of, isColon: typeof Colon.isColon;
    const parseColon: Parser<Slice<Token>, Colon, string, []>;
    class Semicolon implements Equatable, Serializable<Semicolon.JSON> {
        private static _instance;
        static of(): Semicolon;
        private constructor();
        get type(): "semicolon";
        equals(value: unknown): value is this;
        toJSON(): Semicolon.JSON;
        toString(): string;
    }
    namespace Semicolon {
        interface JSON {
            [key: string]: json.JSON;
            type: "semicolon";
        }
        function isSemicolon(value: unknown): value is Semicolon;
    }
    const semicolon: typeof Semicolon.of, isSemicolon: typeof Semicolon.isSemicolon;
    const parseSemicolon: Parser<Slice<Token>, Semicolon, string, []>;
    class Comma implements Equatable, Serializable<Comma.JSON> {
        private static _instance;
        static of(): Comma;
        private constructor();
        get type(): "comma";
        equals(value: unknown): value is this;
        toJSON(): Comma.JSON;
        toString(): string;
    }
    namespace Comma {
        interface JSON {
            [key: string]: json.JSON;
            type: "comma";
        }
        function isComma(value: unknown): value is Comma;
    }
    const comma: typeof Comma.of, isComma: typeof Comma.isComma;
    const parseComma: Parser<Slice<Token>, Comma, string, []>;
    class OpenParenthesis implements Equatable, Serializable<OpenParenthesis.JSON> {
        private static _instance;
        static of(): OpenParenthesis;
        private constructor();
        get type(): "open-parenthesis";
        get mirror(): CloseParenthesis;
        equals(value: unknown): value is this;
        toJSON(): OpenParenthesis.JSON;
        toString(): string;
    }
    namespace OpenParenthesis {
        interface JSON {
            [key: string]: json.JSON;
            type: "open-parenthesis";
        }
        function isOpenParenthesis(value: unknown): value is OpenParenthesis;
    }
    const openParenthesis: typeof OpenParenthesis.of, isOpenParenthesis: typeof OpenParenthesis.isOpenParenthesis;
    const parseOpenParenthesis: Parser<Slice<Token>, OpenParenthesis, string, []>;
    class CloseParenthesis implements Equatable, Serializable<CloseParenthesis.JSON> {
        private static _instance;
        static of(): CloseParenthesis;
        private constructor();
        get type(): "close-parenthesis";
        get mirror(): OpenParenthesis;
        equals(value: unknown): value is this;
        toJSON(): CloseParenthesis.JSON;
        toString(): string;
    }
    namespace CloseParenthesis {
        interface JSON {
            [key: string]: json.JSON;
            type: "close-parenthesis";
        }
        function isCloseParenthesis(value: unknown): value is CloseParenthesis;
    }
    const closeParenthesis: typeof CloseParenthesis.of, isCloseParenthesis: typeof CloseParenthesis.isCloseParenthesis;
    const parseCloseParenthesis: Parser<Slice<Token>, CloseParenthesis, string, []>;
    class OpenSquareBracket implements Equatable, Serializable<OpenSquareBracket.JSON> {
        private static _instance;
        static of(): OpenSquareBracket;
        private constructor();
        get type(): "open-square-bracket";
        get mirror(): CloseSquareBracket;
        equals(value: unknown): value is this;
        toJSON(): OpenSquareBracket.JSON;
        toString(): string;
    }
    namespace OpenSquareBracket {
        interface JSON {
            [key: string]: json.JSON;
            type: "open-square-bracket";
        }
        function isOpenSquareBracket(value: unknown): value is OpenSquareBracket;
    }
    const openSquareBracket: typeof OpenSquareBracket.of, isOpenSquareBracket: typeof OpenSquareBracket.isOpenSquareBracket;
    const parseOpenSquareBracket: Parser<Slice<Token>, OpenSquareBracket, string, []>;
    class CloseSquareBracket implements Equatable, Serializable<CloseSquareBracket.JSON> {
        private static _instance;
        static of(): CloseSquareBracket;
        private constructor();
        get type(): "close-square-bracket";
        get mirror(): OpenSquareBracket;
        equals(value: unknown): value is this;
        toJSON(): CloseSquareBracket.JSON;
        toString(): string;
    }
    namespace CloseSquareBracket {
        interface JSON {
            [key: string]: json.JSON;
            type: "close-square-bracket";
        }
        function isCloseSquareBracket(value: unknown): value is CloseSquareBracket;
    }
    const closeSquareBracket: typeof CloseSquareBracket.of, isCloseSquareBracket: typeof CloseSquareBracket.isCloseSquareBracket;
    const parseCloseSquareBracket: Parser<Slice<Token>, CloseSquareBracket, string, []>;
    class OpenCurlyBracket implements Equatable, Serializable<OpenCurlyBracket.JSON> {
        private static _instance;
        static of(): OpenCurlyBracket;
        private constructor();
        get type(): "open-curly-bracket";
        get mirror(): CloseCurlyBracket;
        equals(value: unknown): value is this;
        toJSON(): OpenCurlyBracket.JSON;
        toString(): string;
    }
    namespace OpenCurlyBracket {
        interface JSON {
            [key: string]: json.JSON;
            type: "open-curly-bracket";
        }
        function isOpenCurlyBracket(value: unknown): value is OpenCurlyBracket;
    }
    const openCurlyBracket: typeof OpenCurlyBracket.of, isOpenCurlyBracket: typeof OpenCurlyBracket.isOpenCurlyBracket;
    const parseOpenCurlyBracket: Parser<Slice<Token>, OpenCurlyBracket, string, []>;
    class CloseCurlyBracket implements Equatable, Serializable<CloseCurlyBracket.JSON> {
        private static _instance;
        static of(): CloseCurlyBracket;
        private constructor();
        get type(): "close-curly-bracket";
        get mirror(): OpenCurlyBracket;
        equals(value: unknown): value is this;
        toJSON(): CloseCurlyBracket.JSON;
        toString(): string;
    }
    namespace CloseCurlyBracket {
        interface JSON {
            [key: string]: json.JSON;
            type: "close-curly-bracket";
        }
        function isCloseCurlyBracket(value: unknown): value is CloseCurlyBracket;
    }
    const closeCurlyBracket: typeof CloseCurlyBracket.of, isCloseCurlyBracket: typeof CloseCurlyBracket.isCloseCurlyBracket;
    const parseCloseCurlyBracket: Parser<Slice<Token>, CloseCurlyBracket, string, []>;
    class OpenComment implements Equatable, Serializable<OpenComment.JSON> {
        private static _instance;
        static of(): OpenComment;
        private constructor();
        get type(): "open-comment";
        equals(value: unknown): value is this;
        toJSON(): OpenComment.JSON;
        toString(): string;
    }
    namespace OpenComment {
        interface JSON {
            [key: string]: json.JSON;
            type: "open-comment";
        }
        function isOpenComment(value: unknown): value is OpenComment;
    }
    const openComment: typeof OpenComment.of, isOpenComment: typeof OpenComment.isOpenComment;
    const parseOpenComment: Parser<Slice<Token>, OpenComment, string, []>;
    class CloseComment implements Equatable, Serializable<CloseComment.JSON> {
        private static _instance;
        static of(): CloseComment;
        private constructor();
        get type(): "close-comment";
        equals(value: unknown): value is this;
        toJSON(): CloseComment.JSON;
        toString(): string;
    }
    namespace CloseComment {
        interface JSON {
            [key: string]: json.JSON;
            type: "close-comment";
        }
        function isCloseComment(value: unknown): value is CloseComment;
    }
    const closeComment: typeof CloseComment.of, isCloseComment: typeof CloseComment.isCloseComment;
    const parseCloseComment: Parser<Slice<Token>, CloseComment, string, []>;
}
//# sourceMappingURL=token.d.ts.map