import type { Hash } from "@siteimprove/alfa-hash";
import { Iterable } from "@siteimprove/alfa-iterable";
import { type Parser as CSSParser } from "../../../../syntax/index.js";
import { Position } from "../../../position/index.js";
import type { PartiallyResolvable, Resolvable } from "../../../resolvable.js";
import { Value } from "../../../value.js";
import { Item } from "../item/index.js";
import { Shape } from "./shape.js";
/**
 * {@link https://drafts.csswg.org/css-images/#radial-gradients}
 *
 * @public
 */
export declare class Radial<I extends Item = Item, S extends Shape = Shape, P extends Position = Position> extends Value<"gradient", Value.HasCalculation<[I, S, P]>> implements Resolvable<Radial.Canonical, Radial.Resolver>, PartiallyResolvable<Radial.PartiallyResolved, Radial.PartialResolver> {
    static of<I extends Item = Item, S extends Shape = Shape, P extends Position = Position>(shape: S, position: P, items: Iterable<I>, repeats: boolean): Radial<I, S, P>;
    private readonly _shape;
    private readonly _position;
    private readonly _items;
    private readonly _repeats;
    private constructor();
    get kind(): "radial";
    get shape(): S;
    get position(): P;
    get items(): Iterable<I>;
    get repeats(): boolean;
    resolve(resolver: Radial.Resolver): Radial.Canonical;
    partiallyResolve(resolver: Radial.PartialResolver): Radial.PartiallyResolved;
    equals(value: Radial): boolean;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): Radial.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Radial {
    type Canonical = Radial<Item.Canonical, Shape.Canonical, Position.Canonical>;
    type PartiallyResolved = Radial<Item.PartiallyResolved, Shape.PartiallyResolved, Position.PartiallyResolved>;
    interface JSON extends Value.JSON<"gradient"> {
        kind: "radial";
        shape: Shape.JSON;
        position: Position.JSON;
        items: Array<Item.JSON>;
        repeats: boolean;
    }
    type Resolver = Item.Resolver & Shape.Resolver & Position.Resolver;
    type PartialResolver = Item.PartialResolver & Shape.PartialResolver & Position.PartialResolver;
    function isRadial(value: unknown): value is Radial;
    /**
     * {@link https://drafts.csswg.org/css-images/#funcdef-radial-gradient}
     */
    const parse: CSSParser<Radial>;
}
//# sourceMappingURL=radial.d.ts.map