import type { Hash } from "@siteimprove/alfa-hash";
import { type Parser as CSSParser } from "../syntax/index.js";
import type { Resolvable } from "./resolvable.js";
import { Value } from "./value.js";
/**
 * {@link https://drafts.csswg.org/css-values/#keywords}
 *
 * @public
 */
export declare class Keyword<T extends string = string> extends Value<"keyword", false> implements Resolvable<Keyword<T>, never> {
    static of<T extends string>(value: T): Keyword<T>;
    private readonly _value;
    private constructor();
    get value(): T;
    resolve(): Keyword<T>;
    is(...values: Array<string>): boolean;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): Keyword.JSON<T>;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Keyword {
    interface JSON<T extends string = string> extends Value.JSON<"keyword"> {
        value: T;
    }
    function isKeyword(value: unknown): value is Keyword;
    /**
     * ToKeywords\<"a" | "b" | "c"\> === Keyword\<"a"\> | Keyword\<"b"\> | Keyword\<"c"\>
     */
    type ToKeywords<Words extends string> = {
        [K in Words]: Keyword<K>;
    }[Words];
    function parse<T extends string>(...keywords: Array<T>): CSSParser<ToKeywords<T>>;
}
//# sourceMappingURL=keyword.d.ts.map