import { Parser } from "@siteimprove/alfa-parser";
import { Predicate } from "@siteimprove/alfa-predicate";
import { Token } from "../syntax/index.js";
import { Value } from "./value.js";
const { map } = Parser;
const { equals } = Predicate;
/**
 * {@link https://drafts.csswg.org/css-values/#keywords}
 *
 * @public
 */
export class Keyword extends Value {
    static of(value) {
        return new Keyword(value);
    }
    _value;
    constructor(value) {
        super("keyword", false);
        this._value = value;
    }
    get value() {
        return this._value;
    }
    resolve() {
        return this;
    }
    is(...values) {
        return values.includes(this._value);
    }
    equals(value) {
        return value instanceof Keyword && value._value === this._value;
    }
    hash(hash) {
        hash.writeString(this._value);
    }
    toJSON() {
        return {
            ...super.toJSON(),
            value: this._value,
        };
    }
    toString() {
        return this._value;
    }
}
/**
 * @public
 */
(function (Keyword) {
    function isKeyword(value) {
        return value instanceof Keyword;
    }
    Keyword.isKeyword = isKeyword;
    function parse(...keywords) {
        return map(Token.parseIdent((ident) => keywords.some(equals(ident.value.toLowerCase()))), (ident) => 
        // Make sure each possible keyword is separated into its own type. For
        // example, we want `parse("foo", "bar")` to result in the type
        // `Keyword<"foo"> | Keyword<"bar">`, not `Keyword<"foo" | "bar">`. Why?
        // Because the former is assignable to the latter, but the latter isn't
        // assignable to the former.
        Keyword.of(ident.value.toLowerCase()));
    }
    Keyword.parse = parse;
})(Keyword || (Keyword = {}));
//# sourceMappingURL=keyword.js.map